@@ -9,9 +9,12 @@
 
 extern crate test;
 
+use std::mem::MaybeUninit;
+
 use ndarray::ShapeBuilder;
 use ndarray::{arr0, arr1, arr2, azip, s};
 use ndarray::{Array, Array1, Array2, Axis, Ix, Zip};
+use ndarray::{Ix1, Ix2, Ix3, Ix5, IxDyn};
 
 use test::black_box;
 
@@ -61,7 +64,7 @@ fn iter_sum_2d_by_row(bench: &mut test::Bencher) {
     let a = black_box(a);
     bench.iter(|| {
         let mut sum = 0;
-        for row in a.genrows() {
+        for row in a.rows() {
             for &elt in row {
                 sum += elt;
             }
@@ -121,7 +124,7 @@ fn iter_sum_2d_cutout_outer_iter(bench: &mut test::Bencher) {
     let a = black_box(av);
     bench.iter(|| {
         let mut sum = 0;
-        for row in a.genrows() {
+        for row in a.rows() {
             for &elt in row {
                 sum += elt;
             }
@@ -253,7 +256,7 @@ fn add_2d_zip(bench: &mut test::Bencher) {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| {
-        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+        Zip::from(&mut a).and(&b).for_each(|a, &b| *a += b);
     });
 }
 
@@ -269,9 +272,9 @@ fn add_2d_alloc_zip_uninit(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| unsafe {
-        let mut c = Array::uninitialized(a.dim());
-        azip!((&a in &a, &b in &b, c in c.raw_view_mut())
-            std::ptr::write(c, a + b)
+        let mut c = Array::<i32, _>::uninit(a.dim());
+        azip!((&a in &a, &b in &b, c in c.raw_view_mut().cast::<i32>())
+            c.write(a + b)
         );
         c
     });
@@ -282,7 +285,7 @@ fn add_2d_alloc_zip_collect(bench: &mut test::Bencher) {
     let a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| {
-        Zip::from(&a).and(&b).apply_collect(|&x, &y| x + y)
+        Zip::from(&a).and(&b).map_collect(|&x, &y| x + y)
     });
 }
 
@@ -298,7 +301,7 @@ fn vec_string_collect(bench: &mut test::Bencher) {
 fn array_string_collect(bench: &mut test::Bencher) {
     let v = Array::from(vec![""; 10240]);
     bench.iter(|| {
-        Zip::from(&v).apply_collect(|s| s.to_owned())
+        Zip::from(&v).map_collect(|s| s.to_owned())
     });
 }
 
@@ -314,7 +317,7 @@ fn vec_f64_collect(bench: &mut test::Bencher) {
 fn array_f64_collect(bench: &mut test::Bencher) {
     let v = Array::from(vec![1.; 10240]);
     bench.iter(|| {
-        Zip::from(&v).apply_collect(|s| s + 1.)
+        Zip::from(&v).map_collect(|s| s + 1.)
     });
 }
 
@@ -348,7 +351,7 @@ fn add_2d_zip_cutout(bench: &mut test::Bencher) {
     let mut acut = a.slice_mut(s![1..-1, 1..-1]);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| {
-        Zip::from(&mut acut).and(&b).apply(|a, &b| *a += b);
+        Zip::from(&mut acut).and(&b).for_each(|a, &b| *a += b);
     });
 }
 
@@ -361,7 +364,7 @@ fn add_2d_cutouts_by_4(bench: &mut test::Bencher) {
     bench.iter(|| {
         Zip::from(a.exact_chunks_mut(chunksz))
             .and(b.exact_chunks(chunksz))
-            .apply(|mut a, b| a += &b);
+            .for_each(|mut a, b| a += &b);
     });
 }
 
@@ -374,7 +377,7 @@ fn add_2d_cutouts_by_16(bench: &mut test::Bencher) {
     bench.iter(|| {
         Zip::from(a.exact_chunks_mut(chunksz))
             .and(b.exact_chunks(chunksz))
-            .apply(|mut a, b| a += &b);
+            .for_each(|mut a, b| a += &b);
     });
 }
 
@@ -387,7 +390,7 @@ fn add_2d_cutouts_by_32(bench: &mut test::Bencher) {
     bench.iter(|| {
         Zip::from(a.exact_chunks_mut(chunksz))
             .and(b.exact_chunks(chunksz))
-            .apply(|mut a, b| a += &b);
+            .for_each(|mut a, b| a += &b);
     });
 }
 
@@ -431,6 +434,22 @@ fn scalar_add_2(bench: &mut test::Bencher) {
     bench.iter(|| n + &a);
 }
 
+#[bench]
+fn scalar_add_strided_1(bench: &mut test::Bencher) {
+    let a =
+        Array::from_shape_fn((64, 64 * 2), |(i, j)| (i * 64 + j) as f32).slice_move(s![.., ..;2]);
+    let n = 1.;
+    bench.iter(|| &a + n);
+}
+
+#[bench]
+fn scalar_add_strided_2(bench: &mut test::Bencher) {
+    let a =
+        Array::from_shape_fn((64, 64 * 2), |(i, j)| (i * 64 + j) as f32).slice_move(s![.., ..;2]);
+    let n = 1.;
+    bench.iter(|| n + &a);
+}
+
 #[bench]
 fn scalar_sub_1(bench: &mut test::Bencher) {
     let a = Array::<f32, _>::zeros((64, 64));
@@ -493,7 +512,7 @@ fn add_2d_zip_strided(bench: &mut test::Bencher) {
     let mut a = a.slice_mut(s![.., ..;2]);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| {
-        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+        Zip::from(&mut a).and(&b).for_each(|a, &b| *a += b);
     });
 }
 
@@ -513,7 +532,7 @@ fn add_2d_zip_one_transposed(bench: &mut test::Bencher) {
     a.swap_axes(0, 1);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     bench.iter(|| {
-        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+        Zip::from(&mut a).and(&b).for_each(|a, &b| *a += b);
     });
 }
 
@@ -535,7 +554,7 @@ fn add_2d_zip_both_transposed(bench: &mut test::Bencher) {
     let mut b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     b.swap_axes(0, 1);
     bench.iter(|| {
-        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+        Zip::from(&mut a).and(&b).for_each(|a, &b| *a += b);
     });
 }
 
@@ -923,3 +942,59 @@ fn sum_axis1(bench: &mut test::Bencher) {
     let a = range_mat(MEAN_SUM_N, MEAN_SUM_N);
     bench.iter(|| a.sum_axis(Axis(1)));
 }
+
+#[bench]
+fn into_dimensionality_ix1_ok(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(Ix1(10));
+    let a = a.view();
+    bench.iter(|| a.into_dimensionality::<Ix1>());
+}
+
+#[bench]
+fn into_dimensionality_ix3_ok(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(Ix3(10, 10, 10));
+    let a = a.view();
+    bench.iter(|| a.into_dimensionality::<Ix3>());
+}
+
+#[bench]
+fn into_dimensionality_ix3_err(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(Ix3(10, 10, 10));
+    let a = a.view();
+    bench.iter(|| a.into_dimensionality::<Ix2>());
+}
+
+#[bench]
+fn into_dimensionality_dyn_to_ix3(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(IxDyn(&[10, 10, 10]));
+    let a = a.view();
+    bench.iter(|| a.clone().into_dimensionality::<Ix3>());
+}
+
+#[bench]
+fn into_dimensionality_dyn_to_dyn(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(IxDyn(&[10, 10, 10]));
+    let a = a.view();
+    bench.iter(|| a.clone().into_dimensionality::<IxDyn>());
+}
+
+#[bench]
+fn into_dyn_ix3(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(Ix3(10, 10, 10));
+    let a = a.view();
+    bench.iter(|| a.into_dyn());
+}
+
+#[bench]
+fn into_dyn_ix5(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(Ix5(2, 2, 2, 2, 2));
+    let a = a.view();
+    bench.iter(|| a.into_dyn());
+}
+
+#[bench]
+fn into_dyn_dyn(bench: &mut test::Bencher) {
+    let a = Array::<f32, _>::zeros(IxDyn(&[10, 10, 10]));
+    let a = a.view();
+    bench.iter(|| a.clone().into_dyn());
+}
