@@ -14,8 +14,9 @@ cargo build --verbose --features "$FEATURES"
 cargo test --verbose --features "$FEATURES"
 cargo test --manifest-path=ndarray-rand/Cargo.toml --no-default-features --verbose
 cargo test --manifest-path=ndarray-rand/Cargo.toml --features quickcheck --verbose
-cargo test --manifest-path=serialization-tests/Cargo.toml --verbose
-cargo test --manifest-path=blas-tests/Cargo.toml --verbose
-CARGO_TARGET_DIR=target/ cargo test --manifest-path=numeric-tests/Cargo.toml --verbose
-([ "$CHANNEL" != "beta" ] || (rustup component add clippy && cargo clippy))
+cargo test --manifest-path=xtest-serialization/Cargo.toml --verbose
+cargo test --manifest-path=xtest-blas/Cargo.toml --verbose
+cargo test --examples
+CARGO_TARGET_DIR=target/ cargo test --manifest-path=xtest-numeric/Cargo.toml --verbose
+CARGO_TARGET_DIR=target/ cargo test --manifest-path=xtest-numeric/Cargo.toml --verbose --features test_blas
 ([ "$CHANNEL" != "nightly" ] || cargo bench --no-run --verbose --features "$FEATURES")
