@@ -1,6 +1,66 @@
 use crate::dimension::IntoDimension;
 use crate::Dimension;
-use crate::{Shape, StrideShape};
+
+/// A contiguous array shape of n dimensions.
+///
+/// Either c- or f- memory ordered (*c* a.k.a *row major* is the default).
+#[derive(Copy, Clone, Debug)]
+pub struct Shape<D> {
+    /// Shape (axis lengths)
+    pub(crate) dim: D,
+    /// Strides can only be C or F here
+    pub(crate) strides: Strides<Contiguous>,
+}
+
+#[derive(Copy, Clone, Debug)]
+pub(crate) enum Contiguous { }
+
+impl<D> Shape<D> {
+    pub(crate) fn is_c(&self) -> bool {
+        matches!(self.strides, Strides::C)
+    }
+}
+
+
+/// An array shape of n dimensions in c-order, f-order or custom strides.
+#[derive(Copy, Clone, Debug)]
+pub struct StrideShape<D> {
+    pub(crate) dim: D,
+    pub(crate) strides: Strides<D>,
+}
+
+/// Stride description
+#[derive(Copy, Clone, Debug)]
+pub(crate) enum Strides<D> {
+    /// Row-major ("C"-order)
+    C,
+    /// Column-major ("F"-order)
+    F,
+    /// Custom strides
+    Custom(D)
+}
+
+impl<D> Strides<D> {
+    /// Return strides for `dim` (computed from dimension if c/f, else return the custom stride)
+    pub(crate) fn strides_for_dim(self, dim: &D) -> D
+        where D: Dimension
+    {
+        match self {
+            Strides::C => dim.default_strides(),
+            Strides::F => dim.fortran_strides(),
+            Strides::Custom(c) => {
+                debug_assert_eq!(c.ndim(), dim.ndim(),
+                    "Custom strides given with {} dimensions, expected {}",
+                    c.ndim(), dim.ndim());
+                c
+            }
+        }
+    }
+
+    pub(crate) fn is_custom(&self) -> bool {
+        matches!(*self, Strides::Custom(_))
+    }
+}
 
 /// A trait for `Shape` and `D where D: Dimension` that allows
 /// customizing the memory layout (strides) of an array shape.
@@ -34,36 +94,18 @@ where
 {
     fn from(value: T) -> Self {
         let shape = value.into_shape();
-        let d = shape.dim;
-        let st = if shape.is_c {
-            d.default_strides()
+        let st = if shape.is_c() {
+            Strides::C
         } else {
-            d.fortran_strides()
+            Strides::F
         };
         StrideShape {
             strides: st,
-            dim: d,
-            custom: false,
+            dim: shape.dim,
         }
     }
 }
 
-/*
-impl<D> From<Shape<D>> for StrideShape<D>
-    where D: Dimension
-{
-    fn from(shape: Shape<D>) -> Self {
-        let d = shape.dim;
-        let st = if shape.is_c { d.default_strides() } else { d.fortran_strides() };
-        StrideShape {
-            strides: st,
-            dim: d,
-            custom: false,
-        }
-    }
-}
-*/
-
 impl<T> ShapeBuilder for T
 where
     T: IntoDimension,
@@ -73,7 +115,7 @@ where
     fn into_shape(self) -> Shape<Self::Dim> {
         Shape {
             dim: self.into_dimension(),
-            is_c: true,
+            strides: Strides::C,
         }
     }
     fn f(self) -> Shape<Self::Dim> {
@@ -93,21 +135,24 @@ where
 {
     type Dim = D;
     type Strides = D;
+
     fn into_shape(self) -> Shape<D> {
         self
     }
+
     fn f(self) -> Self {
         self.set_f(true)
     }
+
     fn set_f(mut self, is_f: bool) -> Self {
-        self.is_c = !is_f;
+        self.strides = if !is_f { Strides::C } else { Strides::F };
         self
     }
+
     fn strides(self, st: D) -> StrideShape<D> {
         StrideShape {
             dim: self.dim,
-            strides: st,
-            custom: true,
+            strides: Strides::Custom(st),
         }
     }
 }
