@@ -8,20 +8,23 @@
 
 #[macro_use]
 mod zipmacro;
+mod ndproducer;
 
+#[cfg(feature = "rayon")]
 use std::mem::MaybeUninit;
 
 use crate::imp_prelude::*;
 use crate::AssignElem;
 use crate::IntoDimension;
 use crate::Layout;
-use crate::NdIndex;
 use crate::partial::Partial;
 
 use crate::indexes::{indices, Indices};
 use crate::layout::{CORDER, FORDER};
 use crate::split_at::{SplitPreference, SplitAt};
 
+pub use self::ndproducer::{NdProducer, IntoNdProducer, Offset};
+
 /// Return if the expression is a break value.
 macro_rules! fold_while {
     ($e:expr) => {
@@ -57,7 +60,8 @@ where
     pub(crate) fn layout_impl(&self) -> Layout {
         let n = self.ndim();
         if self.is_standard_layout() {
-            if n <= 1 {
+            // effectively one-dimensional => C and F layout compatible
+            if n <= 1 || self.shape().iter().filter(|&&len| len > 1).count() <= 1 {
                 Layout::one_dimensional()
             } else {
                 Layout::c()
@@ -91,119 +95,6 @@ where
     private_impl! {}
 }
 
-/// Argument conversion into a producer.
-///
-/// Slices and vectors can be used (equivalent to 1-dimensional array views).
-///
-/// This trait is like `IntoIterator` for `NdProducers` instead of iterators.
-pub trait IntoNdProducer {
-    /// The element produced per iteration.
-    type Item;
-    /// Dimension type of the producer
-    type Dim: Dimension;
-    type Output: NdProducer<Dim = Self::Dim, Item = Self::Item>;
-    /// Convert the value into an `NdProducer`.
-    fn into_producer(self) -> Self::Output;
-}
-
-impl<P> IntoNdProducer for P
-where
-    P: NdProducer,
-{
-    type Item = P::Item;
-    type Dim = P::Dim;
-    type Output = Self;
-    fn into_producer(self) -> Self::Output {
-        self
-    }
-}
-
-/// A producer of an n-dimensional set of elements;
-/// for example an array view, mutable array view or an iterator
-/// that yields chunks.
-///
-/// Producers are used as a arguments to [`Zip`](struct.Zip.html) and
-/// [`azip!()`](macro.azip.html).
-///
-/// # Comparison to `IntoIterator`
-///
-/// Most `NdProducers` are *iterable* (implement `IntoIterator`) but not directly
-/// iterators. This separation is needed because the producer represents
-/// a multidimensional set of items, it can be split along a particular axis for
-/// parallelization, and it has no fixed correspondance to a sequence.
-///
-/// The natural exception is one dimensional producers, like `AxisIter`, which
-/// implement `Iterator` directly
-/// (`AxisIter` traverses a one dimensional sequence, along an axis, while
-/// *producing* multidimensional items).
-///
-/// See also [`IntoNdProducer`](trait.IntoNdProducer.html)
-pub trait NdProducer {
-    /// The element produced per iteration.
-    type Item;
-    // Internal use / Pointee type
-    /// Dimension type
-    type Dim: Dimension;
-
-    // The pointer Ptr is used by an array view to simply point to the
-    // current element. It doesn't have to be a pointer (see Indices).
-    // Its main function is that it can be incremented with a particular
-    // stride (= along a particular axis)
-    #[doc(hidden)]
-    /// Pointer or stand-in for pointer
-    type Ptr: Offset<Stride = Self::Stride>;
-    #[doc(hidden)]
-    /// Pointer stride
-    type Stride: Copy;
-
-    #[doc(hidden)]
-    fn layout(&self) -> Layout;
-    #[doc(hidden)]
-    fn raw_dim(&self) -> Self::Dim;
-    #[doc(hidden)]
-    fn equal_dim(&self, dim: &Self::Dim) -> bool {
-        self.raw_dim() == *dim
-    }
-    #[doc(hidden)]
-    fn as_ptr(&self) -> Self::Ptr;
-    #[doc(hidden)]
-    unsafe fn as_ref(&self, ptr: Self::Ptr) -> Self::Item;
-    #[doc(hidden)]
-    unsafe fn uget_ptr(&self, i: &Self::Dim) -> Self::Ptr;
-    #[doc(hidden)]
-    fn stride_of(&self, axis: Axis) -> <Self::Ptr as Offset>::Stride;
-    #[doc(hidden)]
-    fn contiguous_stride(&self) -> Self::Stride;
-    #[doc(hidden)]
-    fn split_at(self, axis: Axis, index: usize) -> (Self, Self)
-    where
-        Self: Sized;
-
-    private_decl! {}
-}
-
-pub trait Offset: Copy {
-    type Stride: Copy;
-    unsafe fn stride_offset(self, s: Self::Stride, index: usize) -> Self;
-    private_decl! {}
-}
-
-impl<T> Offset for *const T {
-    type Stride = isize;
-    unsafe fn stride_offset(self, s: Self::Stride, index: usize) -> Self {
-        self.offset(s * (index as isize))
-    }
-    private_impl! {}
-}
-
-impl<T> Offset for *mut T {
-    type Stride = isize;
-    unsafe fn stride_offset(self, s: Self::Stride, index: usize) -> Self {
-        self.offset(s * (index as isize))
-    }
-    private_impl! {}
-}
-
 trait ZippableTuple: Sized {
     type Item;
     type Ptr: OffsetTuple<Args = Self::Stride> + Copy;
@@ -217,288 +108,6 @@ trait ZippableTuple: Sized {
     fn split_at(self, axis: Axis, index: usize) -> (Self, Self);
 }
 
-/// An array reference is an n-dimensional producer of element references
-/// (like ArrayView).
-impl<'a, A: 'a, S, D> IntoNdProducer for &'a ArrayBase<S, D>
-where
-    D: Dimension,
-    S: Data<Elem = A>,
-{
-    type Item = &'a A;
-    type Dim = D;
-    type Output = ArrayView<'a, A, D>;
-    fn into_producer(self) -> Self::Output {
-        self.view()
-    }
-}
-
-/// A mutable array reference is an n-dimensional producer of mutable element
-/// references (like ArrayViewMut).
-impl<'a, A: 'a, S, D> IntoNdProducer for &'a mut ArrayBase<S, D>
-where
-    D: Dimension,
-    S: DataMut<Elem = A>,
-{
-    type Item = &'a mut A;
-    type Dim = D;
-    type Output = ArrayViewMut<'a, A, D>;
-    fn into_producer(self) -> Self::Output {
-        self.view_mut()
-    }
-}
-
-/// A slice is a one-dimensional producer
-impl<'a, A: 'a> IntoNdProducer for &'a [A] {
-    type Item = <Self::Output as NdProducer>::Item;
-    type Dim = Ix1;
-    type Output = ArrayView1<'a, A>;
-    fn into_producer(self) -> Self::Output {
-        <_>::from(self)
-    }
-}
-
-/// A mutable slice is a mutable one-dimensional producer
-impl<'a, A: 'a> IntoNdProducer for &'a mut [A] {
-    type Item = <Self::Output as NdProducer>::Item;
-    type Dim = Ix1;
-    type Output = ArrayViewMut1<'a, A>;
-    fn into_producer(self) -> Self::Output {
-        <_>::from(self)
-    }
-}
-
-/// A Vec is a one-dimensional producer
-impl<'a, A: 'a> IntoNdProducer for &'a Vec<A> {
-    type Item = <Self::Output as NdProducer>::Item;
-    type Dim = Ix1;
-    type Output = ArrayView1<'a, A>;
-    fn into_producer(self) -> Self::Output {
-        <_>::from(self)
-    }
-}
-
-/// A mutable Vec is a mutable one-dimensional producer
-impl<'a, A: 'a> IntoNdProducer for &'a mut Vec<A> {
-    type Item = <Self::Output as NdProducer>::Item;
-    type Dim = Ix1;
-    type Output = ArrayViewMut1<'a, A>;
-    fn into_producer(self) -> Self::Output {
-        <_>::from(self)
-    }
-}
-
-impl<'a, A, D: Dimension> NdProducer for ArrayView<'a, A, D> {
-    type Item = &'a A;
-    type Dim = D;
-    type Ptr = *mut A;
-    type Stride = isize;
-
-    private_impl! {}
-    #[doc(hidden)]
-    fn raw_dim(&self) -> Self::Dim {
-        self.raw_dim()
-    }
-
-    #[doc(hidden)]
-    fn equal_dim(&self, dim: &Self::Dim) -> bool {
-        self.dim.equal(dim)
-    }
-
-    #[doc(hidden)]
-    fn as_ptr(&self) -> *mut A {
-        self.as_ptr() as _
-    }
-
-    #[doc(hidden)]
-    fn layout(&self) -> Layout {
-        self.layout_impl()
-    }
-
-    #[doc(hidden)]
-    unsafe fn as_ref(&self, ptr: *mut A) -> Self::Item {
-        &*ptr
-    }
-
-    #[doc(hidden)]
-    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut A {
-        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
-    }
-
-    #[doc(hidden)]
-    fn stride_of(&self, axis: Axis) -> isize {
-        self.stride_of(axis)
-    }
-
-    #[inline(always)]
-    fn contiguous_stride(&self) -> Self::Stride {
-        1
-    }
-
-    #[doc(hidden)]
-    fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
-        self.split_at(axis, index)
-    }
-}
-
-impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
-    type Item = &'a mut A;
-    type Dim = D;
-    type Ptr = *mut A;
-    type Stride = isize;
-
-    private_impl! {}
-    #[doc(hidden)]
-    fn raw_dim(&self) -> Self::Dim {
-        self.raw_dim()
-    }
-
-    #[doc(hidden)]
-    fn equal_dim(&self, dim: &Self::Dim) -> bool {
-        self.dim.equal(dim)
-    }
-
-    #[doc(hidden)]
-    fn as_ptr(&self) -> *mut A {
-        self.as_ptr() as _
-    }
-
-    #[doc(hidden)]
-    fn layout(&self) -> Layout {
-        self.layout_impl()
-    }
-
-    #[doc(hidden)]
-    unsafe fn as_ref(&self, ptr: *mut A) -> Self::Item {
-        &mut *ptr
-    }
-
-    #[doc(hidden)]
-    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut A {
-        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
-    }
-
-    #[doc(hidden)]
-    fn stride_of(&self, axis: Axis) -> isize {
-        self.stride_of(axis)
-    }
-
-    #[inline(always)]
-    fn contiguous_stride(&self) -> Self::Stride {
-        1
-    }
-
-    #[doc(hidden)]
-    fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
-        self.split_at(axis, index)
-    }
-}
-
-impl<A, D: Dimension> NdProducer for RawArrayView<A, D> {
-    type Item = *const A;
-    type Dim = D;
-    type Ptr = *const A;
-    type Stride = isize;
-
-    private_impl! {}
-    #[doc(hidden)]
-    fn raw_dim(&self) -> Self::Dim {
-        self.raw_dim()
-    }
-
-    #[doc(hidden)]
-    fn equal_dim(&self, dim: &Self::Dim) -> bool {
-        self.dim.equal(dim)
-    }
-
-    #[doc(hidden)]
-    fn as_ptr(&self) -> *const A {
-        self.as_ptr()
-    }
-
-    #[doc(hidden)]
-    fn layout(&self) -> Layout {
-        self.layout_impl()
-    }
-
-    #[doc(hidden)]
-    unsafe fn as_ref(&self, ptr: *const A) -> *const A {
-        ptr
-    }
-
-    #[doc(hidden)]
-    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *const A {
-        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
-    }
-
-    #[doc(hidden)]
-    fn stride_of(&self, axis: Axis) -> isize {
-        self.stride_of(axis)
-    }
-
-    #[inline(always)]
-    fn contiguous_stride(&self) -> Self::Stride {
-        1
-    }
-
-    #[doc(hidden)]
-    fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
-        self.split_at(axis, index)
-    }
-}
-
-impl<A, D: Dimension> NdProducer for RawArrayViewMut<A, D> {
-    type Item = *mut A;
-    type Dim = D;
-    type Ptr = *mut A;
-    type Stride = isize;
-
-    private_impl! {}
-    #[doc(hidden)]
-    fn raw_dim(&self) -> Self::Dim {
-        self.raw_dim()
-    }
-
-    #[doc(hidden)]
-    fn equal_dim(&self, dim: &Self::Dim) -> bool {
-        self.dim.equal(dim)
-    }
-
-    #[doc(hidden)]
-    fn as_ptr(&self) -> *mut A {
-        self.as_ptr() as _
-    }
-
-    #[doc(hidden)]
-    fn layout(&self) -> Layout {
-        self.layout_impl()
-    }
-
-    #[doc(hidden)]
-    unsafe fn as_ref(&self, ptr: *mut A) -> *mut A {
-        ptr
-    }
-
-    #[doc(hidden)]
-    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut A {
-        self.ptr.as_ptr().offset(i.index_unchecked(&self.strides))
-    }
-
-    #[doc(hidden)]
-    fn stride_of(&self, axis: Axis) -> isize {
-        self.stride_of(axis)
-    }
-
-    #[inline(always)]
-    fn contiguous_stride(&self) -> Self::Stride {
-        1
-    }
-
-    #[doc(hidden)]
-    fn split_at(self, axis: Axis, index: usize) -> (Self, Self) {
-        self.split_at(axis, index)
-    }
-}
-
 /// Lock step function application across several arrays or other producers.
 ///
 /// Zip allows matching several producers to each other elementwise and applying
@@ -517,7 +126,7 @@ impl<A, D: Dimension> NdProducer for RawArrayViewMut<A, D> {
 /// The order elements are visited is not specified. The producers don’t have to
 /// have the same item type.
 ///
-/// The `Zip` has two methods for function application: `apply` and
+/// The `Zip` has two methods for function application: `for_each` and
 /// `fold_while`. The zip object can be split, which allows parallelization.
 /// A read-only zip object (no mutable producers) can be cloned.
 ///
@@ -545,7 +154,7 @@ impl<A, D: Dimension> NdProducer for RawArrayViewMut<A, D> {
 ///     .and(&b)
 ///     .and(&c)
 ///     .and(&d)
-///     .apply(|w, &x, &y, &z| {
+///     .for_each(|w, &x, &y, &z| {
 ///         *w += x + y * z;
 ///     });
 ///
@@ -561,16 +170,16 @@ impl<A, D: Dimension> NdProducer for RawArrayViewMut<A, D> {
 /// let mut totals = Array1::zeros(a.nrows());
 ///
 /// Zip::from(&mut totals)
-///     .and(a.genrows())
-///     .apply(|totals, row| *totals = row.sum());
+///     .and(a.rows())
+///     .for_each(|totals, row| *totals = row.sum());
 ///
 /// // Check the result against the built in `.sum_axis()` along axis 1.
 /// assert_eq!(totals, a.sum_axis(Axis(1)));
 ///
 ///
-/// // Example 3: Recreate Example 2 using apply_collect to make a new array
+/// // Example 3: Recreate Example 2 using map_collect to make a new array
 ///
-/// let mut totals2 = Zip::from(a.genrows()).apply_collect(|row| row.sum());
+/// let mut totals2 = Zip::from(a.rows()).map_collect(|row| row.sum());
 ///
 /// // Check the result against the previous example.
 /// assert_eq!(totals, totals2);
@@ -691,19 +300,21 @@ impl<P, D> Zip<P, D>
 where
     D: Dimension,
 {
-    fn apply_core<F, Acc>(&mut self, acc: Acc, function: F) -> FoldWhile<Acc>
+    fn for_each_core<F, Acc>(&mut self, acc: Acc, mut function: F) -> FoldWhile<Acc>
     where
         F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
         P: ZippableTuple<Dim = D>,
     {
-        if self.layout.is(CORDER | FORDER) {
-            self.apply_core_contiguous(acc, function)
+        if self.dimension.ndim() == 0 {
+            function(acc, unsafe { self.parts.as_ref(self.parts.as_ptr()) })
+        } else if self.layout.is(CORDER | FORDER) {
+            self.for_each_core_contiguous(acc, function)
         } else {
-            self.apply_core_strided(acc, function)
+            self.for_each_core_strided(acc, function)
         }
     }
 
-    fn apply_core_contiguous<F, Acc>(&mut self, acc: Acc, mut function: F) -> FoldWhile<Acc>
+    fn for_each_core_contiguous<F, Acc>(&mut self, acc: Acc, mut function: F) -> FoldWhile<Acc>
     where
         F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
         P: ZippableTuple<Dim = D>,
@@ -717,7 +328,7 @@ where
         }
     }
 
-    /// The innermost loop of the Zip apply methods
+    /// The innermost loop of the Zip for_each methods
     ///
     /// Run the fold while operation on a stretch of elements with constant strides
     ///
@@ -741,7 +352,7 @@ where
     }
 
 
-    fn apply_core_strided<F, Acc>(&mut self, acc: Acc, function: F) -> FoldWhile<Acc>
+    fn for_each_core_strided<F, Acc>(&mut self, acc: Acc, function: F) -> FoldWhile<Acc>
     where
         F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
         P: ZippableTuple<Dim = D>,
@@ -751,14 +362,14 @@ where
             panic!("Unreachable: ndim == 0 is contiguous")
         }
         if n == 1 || self.layout_tendency >= 0 {
-            self.apply_core_strided_c(acc, function)
+            self.for_each_core_strided_c(acc, function)
         } else {
-            self.apply_core_strided_f(acc, function)
+            self.for_each_core_strided_f(acc, function)
         }
     }
 
     // Non-contiguous but preference for C - unroll over Axis(ndim - 1)
-    fn apply_core_strided_c<F, Acc>(&mut self, mut acc: Acc, mut function: F) -> FoldWhile<Acc>
+    fn for_each_core_strided_c<F, Acc>(&mut self, mut acc: Acc, mut function: F) -> FoldWhile<Acc>
     where
         F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
         P: ZippableTuple<Dim = D>,
@@ -782,7 +393,7 @@ where
     }
 
     // Non-contiguous but preference for F - unroll over Axis(0)
-    fn apply_core_strided_f<F, Acc>(&mut self, mut acc: Acc, mut function: F) -> FoldWhile<Acc>
+    fn for_each_core_strided_f<F, Acc>(&mut self, mut acc: Acc, mut function: F) -> FoldWhile<Acc>
     where
         F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
         P: ZippableTuple<Dim = D>,
@@ -808,10 +419,11 @@ where
         FoldWhile::Continue(acc)
     }
 
+    #[cfg(feature = "rayon")]
     pub(crate) fn uninitalized_for_current_layout<T>(&self) -> Array<MaybeUninit<T>, D>
     {
         let is_f = self.prefer_f();
-        Array::maybe_uninit(self.dimension.clone().set_f(is_f))
+        Array::uninit(self.dimension.clone().set_f(is_f))
     }
 }
 
@@ -936,15 +548,24 @@ macro_rules! map_impl {
         {
             /// Apply a function to all elements of the input arrays,
             /// visiting elements in lock step.
-            pub fn apply<F>(mut self, mut function: F)
+            pub fn for_each<F>(mut self, mut function: F)
                 where F: FnMut($($p::Item),*)
             {
-                self.apply_core((), move |(), args| {
+                self.for_each_core((), move |(), args| {
                     let ($($p,)*) = args;
                     FoldWhile::Continue(function($($p),*))
                 });
             }
 
+            /// Apply a function to all elements of the input arrays,
+            /// visiting elements in lock step.
+            #[deprecated(note="Renamed to .for_each()", since="0.15.0")]
+            pub fn apply<F>(self, function: F)
+                where F: FnMut($($p::Item),*)
+            {
+                self.for_each(function)
+            }
+
             /// Apply a fold function to all elements of the input arrays,
             /// visiting elements in lock step.
             ///
@@ -977,7 +598,7 @@ macro_rules! map_impl {
             where
                 F: FnMut(Acc, $($p::Item),*) -> Acc,
             {
-                self.apply_core(acc, move |acc, args| {
+                self.for_each_core(acc, move |acc, args| {
                     let ($($p,)*) = args;
                     FoldWhile::Continue(function(acc, $($p),*))
                 }).into_inner()
@@ -990,7 +611,7 @@ macro_rules! map_impl {
                 -> FoldWhile<Acc>
                 where F: FnMut(Acc, $($p::Item),*) -> FoldWhile<Acc>
             {
-                self.apply_core(acc, move |acc, args| {
+                self.for_each_core(acc, move |acc, args| {
                     let ($($p,)*) = args;
                     function(acc, $($p),*)
                 })
@@ -1012,7 +633,7 @@ macro_rules! map_impl {
             pub fn all<F>(mut self, mut predicate: F) -> bool
                 where F: FnMut($($p::Item),*) -> bool
             {
-                !self.apply_core((), move |_, args| {
+                !self.for_each_core((), move |_, args| {
                     let ($($p,)*) = args;
                     if predicate($($p),*) {
                         FoldWhile::Continue(())
@@ -1062,42 +683,73 @@ macro_rules! map_impl {
                 }
             }
 
-            /// Apply and collect the results into a new array, which has the same size as the
+            /// Map and collect the results into a new array, which has the same size as the
             /// inputs.
             ///
             /// If all inputs are c- or f-order respectively, that is preserved in the output.
-            pub fn apply_collect<R>(self, f: impl FnMut($($p::Item,)* ) -> R) -> Array<R, D>
+            pub fn map_collect<R>(self, f: impl FnMut($($p::Item,)* ) -> R) -> Array<R, D> {
+                self.map_collect_owned(f)
+            }
+
+            pub(crate) fn map_collect_owned<S, R>(self, f: impl FnMut($($p::Item,)* ) -> R)
+                -> ArrayBase<S, D>
+                where S: DataOwned<Elem = R>
             {
-                // Make uninit result
-                let mut output = self.uninitalized_for_current_layout::<R>();
+                // safe because: all elements are written before the array is completed
 
-                // Use partial to counts the number of filled elements, and can drop the right
-                // number of elements on unwinding (if it happens during apply/collect).
+                let shape = self.dimension.clone().set_f(self.prefer_f());
+                let output = <ArrayBase<S, D>>::build_uninit(shape, |output| {
+                    // Use partial to count the number of filled elements, and can drop the right
+                    // number of elements on unwinding (if it happens during apply/collect).
+                    unsafe {
+                        let output_view = output.cast::<R>();
+                        self.and(output_view)
+                            .collect_with_partial(f)
+                            .release_ownership();
+                    }
+                });
                 unsafe {
-                    let output_view = output.raw_view_mut().cast::<R>();
-                    self.and(output_view)
-                        .collect_with_partial(f)
-                        .release_ownership();
-
                     output.assume_init()
                 }
             }
 
-            /// Apply and assign the results into the producer `into`, which should have the same
+            /// Map and collect the results into a new array, which has the same size as the
+            /// inputs.
+            ///
+            /// If all inputs are c- or f-order respectively, that is preserved in the output.
+            #[deprecated(note="Renamed to .map_collect()", since="0.15.0")]
+            pub fn apply_collect<R>(self, f: impl FnMut($($p::Item,)* ) -> R) -> Array<R, D> {
+                self.map_collect(f)
+            }
+
+            /// Map and assign the results into the producer `into`, which should have the same
             /// size as the other inputs.
             ///
             /// The producer should have assignable items as dictated by the `AssignElem` trait,
             /// for example `&mut R`.
-            pub fn apply_assign_into<R, Q>(self, into: Q, mut f: impl FnMut($($p::Item,)* ) -> R)
+            pub fn map_assign_into<R, Q>(self, into: Q, mut f: impl FnMut($($p::Item,)* ) -> R)
                 where Q: IntoNdProducer<Dim=D>,
                       Q::Item: AssignElem<R>
             {
                 self.and(into)
-                    .apply(move |$($p, )* output_| {
+                    .for_each(move |$($p, )* output_| {
                         output_.assign_elem(f($($p ),*));
                     });
             }
 
+            /// Map and assign the results into the producer `into`, which should have the same
+            /// size as the other inputs.
+            ///
+            /// The producer should have assignable items as dictated by the `AssignElem` trait,
+            /// for example `&mut R`.
+            #[deprecated(note="Renamed to .map_assign_into()", since="0.15.0")]
+            pub fn apply_assign_into<R, Q>(self, into: Q, f: impl FnMut($($p::Item,)* ) -> R)
+                where Q: IntoNdProducer<Dim=D>,
+                      Q::Item: AssignElem<R>
+            {
+                self.map_assign_into(into, f)
+            }
+
 
             );
 
@@ -1119,7 +771,7 @@ macro_rules! map_impl {
                       $($p: NdProducer<Dim=D> ,)*
                       PLast: NdProducer<Dim = D, Item = *mut R, Ptr = *mut R, Stride = isize>,
             {
-                /// The inner workings of apply_collect and par_apply_collect
+                /// The inner workings of map_collect and par_map_collect
                 ///
                 /// Apply the function and collect the results into the output (last producer)
                 /// which should be a raw array view; a Partial that owns the written
@@ -1140,14 +792,25 @@ macro_rules! map_impl {
                 {
                     // Get the last producer; and make a Partial that aliases its data pointer
                     let (.., ref output) = &self.parts;
-                    debug_assert!(output.layout().is(CORDER | FORDER));
-                    debug_assert_eq!(output.layout().tendency() >= 0, self.layout_tendency >= 0);
+
+                    // debug assert that the output is contiguous in the memory layout we need
+                    if cfg!(debug_assertions) {
+                        let out_layout = output.layout();
+                        assert!(out_layout.is(CORDER | FORDER));
+                        assert!(
+                            (self.layout_tendency <= 0 && out_layout.tendency() <= 0) ||
+                            (self.layout_tendency >= 0 && out_layout.tendency() >= 0),
+                            "layout tendency violation for self layout {:?}, output layout {:?},\
+                            output shape {:?}",
+                            self.layout, out_layout, output.raw_dim());
+                    }
+
                     let mut partial = Partial::new(output.as_ptr());
 
                     // Apply the mapping function on this zip
                     // if we panic with unwinding; Partial will drop the written elements.
                     let partial_len = &mut partial.len;
-                    self.apply(move |$($p,)* output_elem: *mut R| {
+                    self.for_each(move |$($p,)* output_elem: *mut R| {
                         output_elem.write(f($($p),*));
                         if std::mem::needs_drop::<R>() {
                             *partial_len += 1;
