@@ -51,6 +51,8 @@ fn test_arcarray_thread_safe() {
 }
 
 #[test]
+#[cfg(feature = "std")]
+#[allow(deprecated)] // uninitialized
 fn test_uninit() {
     unsafe {
         let mut a = Array::<f32, _>::uninitialized((3, 4).f());
@@ -148,6 +150,7 @@ fn test_from_fn_f3() {
 fn deny_wraparound_from_vec() {
     let five = vec![0; 5];
     let five_large = Array::from_shape_vec((3, 7, 29, 36760123, 823996703), five.clone());
+    println!("{:?}", five_large);
     assert!(five_large.is_err());
     let six = Array::from_shape_vec(6, five.clone());
     assert!(six.is_err());
@@ -190,12 +193,19 @@ fn deny_wraparound_from_shape_fn() {
 
 #[should_panic]
 #[test]
-fn deny_wraparound_uninit() {
+#[allow(deprecated)] // uninitialized
+fn deny_wraparound_uninitialized() {
     unsafe {
         let _five_large = Array::<f32, _>::uninitialized((3, 7, 29, 36760123, 823996703));
     }
 }
 
+#[should_panic]
+#[test]
+fn deny_wraparound_uninit() {
+    let _five_large = Array::<f32, _>::uninit((3, 7, 29, 36760123, 823996703));
+}
+
 
 #[test]
 fn maybe_uninit_1() {
@@ -203,18 +213,18 @@ fn maybe_uninit_1() {
 
     unsafe {
         // Array
-        type Mat<D> = Array<MaybeUninit<f32>, D>;
+        type Mat<D> = Array<f32, D>;
 
-        let mut a = Mat::maybe_uninit((10, 10));
+        let mut a = Mat::uninit((10, 10));
         a.mapv_inplace(|_| MaybeUninit::new(1.));
 
         let a_init = a.assume_init();
         assert_eq!(a_init, Array2::from_elem(a_init.dim(), 1.));
 
         // ArcArray
-        type ArcMat<D> = ArcArray<MaybeUninit<f32>, D>;
+        type ArcMat<D> = ArcArray<f32, D>;
 
-        let mut a = ArcMat::maybe_uninit((10, 10));
+        let mut a = ArcMat::uninit((10, 10));
         a.mapv_inplace(|_| MaybeUninit::new(1.));
         let a2 = a.clone();
 
@@ -226,15 +236,15 @@ fn maybe_uninit_1() {
         assert_eq!(av_init, Array2::from_elem(a_init.dim(), 1.));
 
         // RawArrayViewMut
-        let mut a = Mat::maybe_uninit((10, 10));
+        let mut a = Mat::uninit((10, 10));
         let v = a.raw_view_mut();
         Zip::from(v)
-            .apply(|ptr| *(*ptr).as_mut_ptr() = 1.);
+            .for_each(|ptr| *(*ptr).as_mut_ptr() = 1.);
 
         let u = a.raw_view_mut().assume_init();
 
         Zip::from(u)
-            .apply(|ptr| assert_eq!(*ptr, 1.));
+            .for_each(|ptr| assert_eq!(*ptr, 1.));
 
     }
 }
