@@ -9,11 +9,11 @@
 
 use defmac::defmac;
 use itertools::{enumerate, zip, Itertools};
-use ndarray::indices;
 use ndarray::prelude::*;
 use ndarray::{arr3, rcarr2};
-use ndarray::{Slice, SliceInfo, SliceOrIndex};
-use std::iter::FromIterator;
+use ndarray::indices;
+use ndarray::{Slice, SliceInfo, SliceInfoElem};
+use std::convert::TryFrom;
 
 macro_rules! assert_panics {
     ($body:expr) => {
@@ -67,6 +67,7 @@ fn arrayviewmut_shrink_lifetime<'a, 'b: 'a>(
 }
 
 #[test]
+#[cfg(feature = "std")]
 fn test_mat_mul() {
     // smoke test, a big matrix multiplication of uneven size
     let (n, m) = (45, 33);
@@ -85,8 +86,8 @@ fn test_slice() {
         *elt = i;
     }
 
-    let vi = A.slice(s![1.., ..;2, Slice::new(0, None, 2)]);
-    assert_eq!(vi.shape(), &[2, 2, 3]);
+    let vi = A.slice(s![1.., ..;2, NewAxis, Slice::new(0, None, 2)]);
+    assert_eq!(vi.shape(), &[2, 2, 1, 3]);
     let vi = A.slice(s![.., .., ..]);
     assert_eq!(vi.shape(), A.shape());
     assert!(vi.iter().zip(A.iter()).all(|(a, b)| a == b));
@@ -138,8 +139,8 @@ fn test_slice_with_many_dim() {
         *elt = i;
     }
 
-    let vi = A.slice(s![..2, .., ..;2, ..1, ..1, 1.., ..]);
-    let new_shape = &[2, 1, 2, 1, 1, 1, 1][..];
+    let vi = A.slice(s![..2, NewAxis, .., ..;2, NewAxis, ..1, ..1, 1.., ..]);
+    let new_shape = &[2, 1, 1, 2, 1, 1, 1, 1, 1][..];
     assert_eq!(vi.shape(), new_shape);
     let correct = array![
         [A[&[0, 0, 0, 0, 0, 1, 0][..]], A[&[0, 0, 2, 0, 0, 1, 0][..]]],
@@ -196,105 +197,141 @@ fn test_slice_args_eval_step_once() {
 #[test]
 fn test_slice_array_fixed() {
     let mut arr = Array3::<f64>::zeros((5, 2, 5));
-    let info = s![1.., 1, ..;2];
+    let info = s![1.., 1, NewAxis, ..;2];
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_collapse(info);
+    let info2 = s![1.., 1, ..;2];
+    arr.view().slice_collapse(info2);
 }
 
 #[test]
 fn test_slice_dyninput_array_fixed() {
     let mut arr = Array3::<f64>::zeros((5, 2, 5)).into_dyn();
-    let info = s![1.., 1, ..;2];
+    let info = s![1.., 1, NewAxis, ..;2];
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_collapse(info.as_ref());
+    let info2 = s![1.., 1, ..;2];
+    arr.view().slice_collapse(info2);
 }
 
 #[test]
 fn test_slice_array_dyn() {
     let mut arr = Array3::<f64>::zeros((5, 2, 5));
-    let info = &SliceInfo::<_, IxDyn>::new([
-        SliceOrIndex::from(1..),
-        SliceOrIndex::from(1),
-        SliceOrIndex::from(..).step_by(2),
+    let info = SliceInfo::<_, Ix3, IxDyn>::try_from([
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(NewAxis),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
     ])
     .unwrap();
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_collapse(info);
+    let info2 = SliceInfo::<_, Ix3, IxDyn>::try_from([
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
+    ])
+    .unwrap();
+    arr.view().slice_collapse(info2);
 }
 
 #[test]
 fn test_slice_dyninput_array_dyn() {
     let mut arr = Array3::<f64>::zeros((5, 2, 5)).into_dyn();
-    let info = &SliceInfo::<_, IxDyn>::new([
-        SliceOrIndex::from(1..),
-        SliceOrIndex::from(1),
-        SliceOrIndex::from(..).step_by(2),
+    let info = SliceInfo::<_, Ix3, IxDyn>::try_from([
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(NewAxis),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
     ])
     .unwrap();
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_collapse(info.as_ref());
+    let info2 = SliceInfo::<_, Ix3, IxDyn>::try_from([
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
+    ])
+    .unwrap();
+    arr.view().slice_collapse(info2);
 }
 
 #[test]
 fn test_slice_dyninput_vec_fixed() {
     let mut arr = Array3::<f64>::zeros((5, 2, 5)).into_dyn();
-    let info = &SliceInfo::<_, Ix2>::new(vec![
-        SliceOrIndex::from(1..),
-        SliceOrIndex::from(1),
-        SliceOrIndex::from(..).step_by(2),
+    let info = &SliceInfo::<_, Ix3, Ix3>::try_from(vec![
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(NewAxis),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
     ])
     .unwrap();
-    arr.slice(info.as_ref());
-    arr.slice_mut(info.as_ref());
-    arr.view().slice_move(info.as_ref());
-    arr.view().slice_collapse(info.as_ref());
+    arr.slice(info);
+    arr.slice_mut(info);
+    arr.view().slice_move(info);
+    let info2 = SliceInfo::<_, Ix3, Ix2>::try_from(vec![
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
+    ])
+    .unwrap();
+    arr.view().slice_collapse(info2);
 }
 
 #[test]
 fn test_slice_dyninput_vec_dyn() {
     let mut arr = Array3::<f64>::zeros((5, 2, 5)).into_dyn();
-    let info = &SliceInfo::<_, IxDyn>::new(vec![
-        SliceOrIndex::from(1..),
-        SliceOrIndex::from(1),
-        SliceOrIndex::from(..).step_by(2),
+    let info = &SliceInfo::<_, Ix3, IxDyn>::try_from(vec![
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(NewAxis),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
     ])
     .unwrap();
-    arr.slice(info.as_ref());
-    arr.slice_mut(info.as_ref());
-    arr.view().slice_move(info.as_ref());
-    arr.view().slice_collapse(info.as_ref());
+    arr.slice(info);
+    arr.slice_mut(info);
+    arr.view().slice_move(info);
+    let info2 = SliceInfo::<_, Ix3, IxDyn>::try_from(vec![
+        SliceInfoElem::from(1..),
+        SliceInfoElem::from(1),
+        SliceInfoElem::from(Slice::from(..).step_by(2)),
+    ])
+    .unwrap();
+    arr.view().slice_collapse(info2);
 }
 
 #[test]
-fn test_slice_with_subview() {
+fn test_slice_with_subview_and_new_axis() {
     let mut arr = ArcArray::<usize, _>::zeros((3, 5, 4));
     for (i, elt) in arr.iter_mut().enumerate() {
         *elt = i;
     }
 
-    let vi = arr.slice(s![1.., 2, ..;2]);
-    assert_eq!(vi.shape(), &[2, 2]);
+    let vi = arr.slice(s![NewAxis, 1.., 2, ..;2]);
+    assert_eq!(vi.shape(), &[1, 2, 2]);
     assert!(vi
         .iter()
-        .zip(arr.index_axis(Axis(1), 2).slice(s![1.., ..;2]).iter())
+        .zip(
+            arr.index_axis(Axis(1), 2)
+                .slice(s![1.., ..;2])
+                .insert_axis(Axis(0))
+                .iter()
+        )
         .all(|(a, b)| a == b));
 
-    let vi = arr.slice(s![1, 2, ..;2]);
-    assert_eq!(vi.shape(), &[2]);
+    let vi = arr.slice(s![1, NewAxis, 2, ..;2]);
+    assert_eq!(vi.shape(), &[1, 2]);
     assert!(vi
         .iter()
         .zip(
             arr.index_axis(Axis(0), 1)
                 .index_axis(Axis(0), 2)
                 .slice(s![..;2])
+                .insert_axis(Axis(0))
                 .iter()
         )
         .all(|(a, b)| a == b));
@@ -342,6 +379,13 @@ fn test_slice_collapse_with_indices() {
     assert_eq!(vi, Array3::from_elem((1, 1, 1), elem));
 }
 
+#[test]
+#[should_panic]
+fn test_slice_collapse_with_newaxis() {
+    let mut arr = Array2::<u8>::zeros((2, 3));
+    arr.slice_collapse(s![0, 0, NewAxis]);
+}
+
 #[test]
 fn test_multislice() {
     macro_rules! do_test {
@@ -382,7 +426,7 @@ fn test_multislice() {
 fn test_multislice_intersecting() {
     assert_panics!({
         let mut arr = Array2::<u8>::zeros((8, 6));
-        arr.multi_slice_mut((s![3, ..], s![3, ..]));
+        arr.multi_slice_mut((s![3, .., NewAxis], s![3, ..]));
     });
     assert_panics!({
         let mut arr = Array2::<u8>::zeros((8, 6));
@@ -390,7 +434,7 @@ fn test_multislice_intersecting() {
     });
     assert_panics!({
         let mut arr = Array2::<u8>::zeros((8, 6));
-        arr.multi_slice_mut((s![3, ..], s![..;3, ..]));
+        arr.multi_slice_mut((s![3, ..], s![..;3, NewAxis, ..]));
     });
     assert_panics!({
         let mut arr = Array2::<u8>::zeros((8, 6));
@@ -609,6 +653,7 @@ fn test_cow_shrink() {
 }
 
 #[test]
+#[cfg(feature = "std")]
 fn test_sub() {
     let mat = ArcArray::linspace(0., 15., 16).reshape((2, 4, 2));
     let s1 = mat.index_axis(Axis(0), 0);
@@ -623,6 +668,7 @@ fn test_sub() {
 
 #[should_panic]
 #[test]
+#[cfg(feature = "std")]
 fn test_sub_oob_1() {
     let mat = ArcArray::linspace(0., 15., 16).reshape((2, 4, 2));
     mat.index_axis(Axis(0), 2);
@@ -854,31 +900,6 @@ fn standard_layout() {
     assert!(x4.is_standard_layout());
 }
 
-#[test]
-fn assign() {
-    let mut a = arr2(&[[1., 2.], [3., 4.]]);
-    let b = arr2(&[[1., 3.], [2., 4.]]);
-    a.assign(&b);
-    assert_eq!(a, b);
-
-    /* Test broadcasting */
-    a.assign(&ArcArray::zeros(1));
-    assert_eq!(a, ArcArray::zeros((2, 2)));
-
-    /* Test other type */
-    a.assign(&Array::from_elem((2, 2), 3.));
-    assert_eq!(a, ArcArray::from_elem((2, 2), 3.));
-
-    /* Test mut view */
-    let mut a = arr2(&[[1, 2], [3, 4]]);
-    {
-        let mut v = a.view_mut();
-        v.slice_collapse(s![..1, ..]);
-        v.fill(0);
-    }
-    assert_eq!(a, arr2(&[[0, 0], [3, 4]]));
-}
-
 #[test]
 fn iter_size_hint() {
     let mut a = arr2(&[[1., 2.], [3., 4.]]);
@@ -922,7 +943,7 @@ fn zero_axes() {
     }
     a.map(|_| panic!());
     a.map_inplace(|_| panic!());
-    a.visit(|_| panic!());
+    a.for_each(|_| panic!());
     println!("{:?}", a);
     let b = arr2::<f32, _>(&[[], [], [], []]);
     println!("{:?}\n{:?}", b.shape(), b);
@@ -1563,6 +1584,49 @@ fn arithmetic_broadcast() {
     a.swap_axes(0, 1);
     let b = a.clone() / aview0(&1.);
     assert_eq!(a, b);
+
+    // reference
+    let a = arr2(&[[2], [3], [4]]);
+    let b = arr1(&[5, 6, 7]);
+    assert_eq!(&a + &b, arr2(&[[7, 8, 9], [8, 9, 10], [9, 10, 11]]));
+    assert_eq!(
+        a.clone() - &b,
+        arr2(&[[-3, -4, -5], [-2, -3, -4], [-1, -2, -3]])
+    );
+    assert_eq!(
+        a.clone() * b.clone(),
+        arr2(&[[10, 12, 14], [15, 18, 21], [20, 24, 28]])
+    );
+    assert_eq!(&b / a, arr2(&[[2, 3, 3], [1, 2, 2], [1, 1, 1]]));
+
+    // Negative strides and non-contiguous memory
+    let s = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12];
+    let s = Array3::from_shape_vec((2, 3, 2).strides((1, 4, 2)), s.to_vec()).unwrap();
+    let a = s.slice(s![..;-1,..;2,..]);
+    let b = s.slice(s![..2, -1, ..]);
+    let mut c = s.clone();
+    c.collapse_axis(Axis(2), 1);
+    let c = c.slice(s![1,..;2,..]);
+    assert_eq!(
+        &a.to_owned() + &b,
+        arr3(&[[[11, 15], [20, 24]], [[10, 14], [19, 23]]])
+    );
+    assert_eq!(
+        &a + b.into_owned() + c,
+        arr3(&[[[15, 19], [32, 36]], [[14, 18], [31, 35]]])
+    );
+
+    // shared array
+    let sa = a.to_shared();
+    let sa2 = sa.to_shared();
+    let sb = b.to_shared();
+    let sb2 = sb.to_shared();
+    let sc = c.to_shared();
+    let sc2 = sc.into_shared();
+    assert_eq!(
+        sa2 + &sb2 + sc2.into_owned(),
+        arr3(&[[[15, 19], [32, 36]], [[14, 18], [31, 35]]])
+    );
 }
 
 #[test]
@@ -1610,6 +1674,7 @@ fn scalar_ops() {
 }
 
 #[test]
+#[cfg(feature = "std")]
 fn split_at() {
     let mut a = arr2(&[[1., 2.], [3., 4.]]);
 
@@ -1661,6 +1726,7 @@ fn deny_split_at_index_out_of_bounds() {
 }
 
 #[test]
+#[cfg(feature = "std")]
 fn test_range() {
     let a = Array::range(0., 5., 1.);
     assert_eq!(a.len(), 5);
@@ -1701,7 +1767,7 @@ fn test_f_order() {
     assert_eq!(c.strides(), &[3, 1]);
     assert_eq!(f.strides(), &[1, 2]);
     itertools::assert_equal(f.iter(), c.iter());
-    itertools::assert_equal(f.genrows(), c.genrows());
+    itertools::assert_equal(f.rows(), c.rows());
     itertools::assert_equal(f.outer_iter(), c.outer_iter());
     itertools::assert_equal(f.axis_iter(Axis(0)), c.axis_iter(Axis(0)));
     itertools::assert_equal(f.axis_iter(Axis(1)), c.axis_iter(Axis(1)));
@@ -1717,10 +1783,18 @@ fn to_owned_memory_order() {
     // input.
     let c = arr2(&[[1, 2, 3], [4, 5, 6]]);
     let mut f = c.view();
+
+    // transposed array
     f.swap_axes(0, 1);
     let fo = f.to_owned();
     assert_eq!(f, fo);
     assert_eq!(f.strides(), fo.strides());
+
+    // negated stride axis
+    f.invert_axis(Axis(1));
+    let fo2 = f.to_owned();
+    assert_eq!(f, fo2);
+    assert_eq!(f.strides(), fo2.strides());
 }
 
 #[test]
@@ -1729,6 +1803,7 @@ fn to_owned_neg_stride() {
     c.slice_collapse(s![.., ..;-1]);
     let co = c.to_owned();
     assert_eq!(c, co);
+    assert_eq!(c.strides(), co.strides());
 }
 
 #[test]
@@ -1752,6 +1827,27 @@ fn map_memory_order() {
     assert_eq!(amap.strides(), v.strides());
 }
 
+#[test]
+fn test_view_from_shape() {
+    let s = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11];
+    let a = ArrayView::from_shape((2, 3, 2), &s).unwrap();
+    let mut answer = Array::from(s.to_vec()).into_shape((2, 3, 2)).unwrap();
+    assert_eq!(a, answer);
+
+    // custom strides (row major)
+    let a = ArrayView::from_shape((2, 3, 2).strides((6, 2, 1)), &s).unwrap();
+    assert_eq!(a, answer);
+
+    // custom strides (col major)
+    let a = ArrayView::from_shape((2, 3, 2).strides((1, 2, 6)), &s).unwrap();
+    assert_eq!(a, answer.t());
+
+    // negative strides
+    let a = ArrayView::from_shape((2, 3, 2).strides((6, (-2isize) as usize, 1)), &s).unwrap();
+    answer.invert_axis(Axis(1));
+    assert_eq!(a, answer);
+}
+
 #[test]
 fn test_contiguous() {
     let c = arr3(&[[[1, 2, 3], [4, 5, 6]], [[4, 5, 6], [7, 7, 7]]]);
@@ -1787,14 +1883,61 @@ fn test_contiguous() {
 }
 
 #[test]
-#[allow(deprecated)]
-fn test_all_close() {
-    let c = arr3(&[
-        [[1., 2., 3.], [1.5, 1.5, 3.]],
-        [[1., 2., 3.], [1., 2.5, 3.]],
-    ]);
-    assert!(c.all_close(&aview1(&[1., 2., 3.]), 1.));
-    assert!(!c.all_close(&aview1(&[1., 2., 3.]), 0.1));
+fn test_contiguous_neg_strides() {
+    let s = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13];
+    let a = ArrayView::from_shape((2, 3, 2).strides((1, 4, 2)), &s).unwrap();
+    assert_eq!(
+        a,
+        arr3(&[[[0, 2], [4, 6], [8, 10]], [[1, 3], [5, 7], [9, 11]]])
+    );
+    assert!(a.as_slice_memory_order().is_some());
+
+    let mut b = a.slice(s![..;1, ..;-1, ..;-1]);
+    assert_eq!(
+        b,
+        arr3(&[[[10, 8], [6, 4], [2, 0]], [[11, 9], [7, 5], [3, 1]]])
+    );
+    assert!(b.as_slice_memory_order().is_some());
+
+    b.swap_axes(1, 2);
+    assert_eq!(b, arr3(&[[[10, 6, 2], [8, 4, 0]], [[11, 7, 3], [9, 5, 1]]]));
+    assert!(b.as_slice_memory_order().is_some());
+
+    b.invert_axis(Axis(0));
+    assert_eq!(b, arr3(&[[[11, 7, 3], [9, 5, 1]], [[10, 6, 2], [8, 4, 0]]]));
+    assert!(b.as_slice_memory_order().is_some());
+
+    let mut c = b.reversed_axes();
+    assert_eq!(
+        c,
+        arr3(&[[[11, 10], [9, 8]], [[7, 6], [5, 4]], [[3, 2], [1, 0]]])
+    );
+    assert!(c.as_slice_memory_order().is_some());
+
+    c.merge_axes(Axis(1), Axis(2));
+    assert_eq!(c, arr3(&[[[11, 10, 9, 8]], [[7, 6, 5, 4]], [[3, 2, 1, 0]]]));
+    assert!(c.as_slice_memory_order().is_some());
+
+    let d = b.remove_axis(Axis(1));
+    assert_eq!(d, arr2(&[[11, 7, 3], [10, 6, 2]]));
+    assert!(d.as_slice_memory_order().is_none());
+
+    let e = b.remove_axis(Axis(2));
+    assert_eq!(e, arr2(&[[11, 9], [10, 8]]));
+    assert!(e.as_slice_memory_order().is_some());
+
+    let f = e.insert_axis(Axis(2));
+    assert_eq!(f, arr3(&[[[11], [9]], [[10], [8]]]));
+    assert!(f.as_slice_memory_order().is_some());
+
+    let mut g = b.clone();
+    g.collapse_axis(Axis(1), 0);
+    assert_eq!(g, arr3(&[[[11, 7, 3]], [[10, 6, 2]]]));
+    assert!(g.as_slice_memory_order().is_none());
+
+    b.collapse_axis(Axis(2), 0);
+    assert_eq!(b, arr3(&[[[11], [9]], [[10], [8]]]));
+    assert!(b.as_slice_memory_order().is_some());
 }
 
 #[test]
@@ -1851,6 +1994,46 @@ fn test_view_from_shape_ptr() {
     assert_eq!(view, aview2(&[[0, 0, 2], [3, 4, 6]]));
 }
 
+#[should_panic(expected = "Unsupported")]
+#[cfg(debug_assertions)]
+#[test]
+fn test_view_from_shape_ptr_deny_neg_strides() {
+    let data = [0, 1, 2, 3, 4, 5];
+    let _view = unsafe {
+        ArrayView::from_shape_ptr((2, 3).strides((-3isize as usize, 1)), data.as_ptr())
+    };
+}
+
+#[should_panic(expected = "Unsupported")]
+#[cfg(debug_assertions)]
+#[test]
+fn test_view_mut_from_shape_ptr_deny_neg_strides() {
+    let mut data = [0, 1, 2, 3, 4, 5];
+    let _view = unsafe {
+        ArrayViewMut::from_shape_ptr((2, 3).strides((-3isize as usize, 1)), data.as_mut_ptr())
+    };
+}
+
+#[should_panic(expected = "Unsupported")]
+#[cfg(debug_assertions)]
+#[test]
+fn test_raw_view_from_shape_ptr_deny_neg_strides() {
+    let data = [0, 1, 2, 3, 4, 5];
+    let _view = unsafe {
+        RawArrayView::from_shape_ptr((2, 3).strides((-3isize as usize, 1)), data.as_ptr())
+    };
+}
+
+#[should_panic(expected = "Unsupported")]
+#[cfg(debug_assertions)]
+#[test]
+fn test_raw_view_mut_from_shape_ptr_deny_neg_strides() {
+    let mut data = [0, 1, 2, 3, 4, 5];
+    let _view = unsafe {
+        RawArrayViewMut::from_shape_ptr((2, 3).strides((-3isize as usize, 1)), data.as_mut_ptr())
+    };
+}
+
 #[test]
 fn test_default() {
     let a = <Array<f32, Ix2> as Default>::default();
